<?php

namespace TVn7\EventBundle\Entity;

use Doctrine\ORM\Mapping as ORM;

use TVn7\EventBundle\Entity\Comment;
use TVn7\EventBundle\Entity\Event;
use TVn7\EventBundle\Entity\Master;
use TVn7\EventBundle\Entity\Clip;
use TVn7\EventBundle\Entity\Sequence;
use TVn7\UserBundle\Entity\Membre;

use Gedmo\Mapping\Annotation as Gedmo;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Symfony\Component\Validator\Context\ExecutionContextInterface;

/**
 * Music
 *
 * @ORM\Table(name="tdb_event_music")
 * @ORM\Entity(repositoryClass="TVn7\EventBundle\Repository\MusicRepository")
 * @UniqueEntity(fields={"name","author"},message="Ce couple nom/artiste est déjà utilisé.")
 * @UniqueEntity(fields="url",message="Cette url est déjà enregistrée.")
 * @Gedmo\Loggable(logEntryClass="TVn7\LogBundle\Entity\AppsLog")
 */
class Music
{
    /**
     * @var int
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(name="name", type="string", length=255)
     * @Assert\NotBlank()
     * @Assert\Length(min = 2, max = 50)
     * @Assert\Type(type="string")
     * @Gedmo\Versioned
     */
    private $name;

    /**
     * @var string
     * @Gedmo\Slug(fields={"author","name"}, updatable=false)
     * @ORM\Column(name="slug", type="string", length=255, unique=true)
     */
    private $slug;

    /**
     * @var string
     *
     * @ORM\Column(name="author", type="string", length=255)
     * @Assert\NotBlank()
     * @Assert\Length(min = 2, max = 50)
     * @Assert\Type(type="string")
     * @Gedmo\Versioned
     */
    private $author;

    /**
     * @var string
     *
     * @ORM\Column(name="url", type="string", length=255, nullable=true)
     * @Assert\Url()
     * @Gedmo\Versioned
     */
    private $url;

    /**
     * @var Clip
     * @ORM\ManyToMany(targetEntity="TVn7\EventBundle\Entity\Clip", mappedBy="musics")
     */
    private $clips;
    
    /**
     * Constructor
     */
    public function __construct()
    {
        $this->clips = new \Doctrine\Common\Collections\ArrayCollection();
    }

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set name
     *
     * @param string $name
     *
     * @return Music
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set slug
     *
     * @param string $slug
     *
     * @return Music
     */
    public function setSlug($slug)
    {
        $this->slug = $slug;

        return $this;
    }

    /**
     * Get slug
     *
     * @return string
     */
    public function getSlug()
    {
        return $this->slug;
    }

    /**
     * Set author
     *
     * @param string $author
     *
     * @return Music
     */
    public function setAuthor($author)
    {
        $this->author = $author;

        return $this;
    }

    /**
     * Get author
     *
     * @return string
     */
    public function getAuthor()
    {
        return $this->author;
    }

    /**
     * Set url
     *
     * @param string $url
     *
     * @return Music
     */
    public function setUrl($url)
    {
        $this->url = $url;

        return $this;
    }

    /**
     * Get url
     *
     * @return string
     */
    public function getUrl()
    {
        return $this->url;
    }

    /**
     * Add clip
     *
     * @param \TVn7\EventBundle\Entity\Clip $clip
     *
     * @return Music
     */
    public function addClip(\TVn7\EventBundle\Entity\Clip $clip)
    {
        $this->clips[] = $clip;

        return $this;
    }

    /**
     * Remove clip
     *
     * @param \TVn7\EventBundle\Entity\Clip $clip
     */
    public function removeClip(\TVn7\EventBundle\Entity\Clip $clip)
    {
        $this->clips->removeElement($clip);
    }

    /**
     * Get clips
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getClips()
    {
        return $this->clips;
    }
    
    /**
     * 
     * @return string
     */
    public function __toString() {
        return $this->getAuthor() .' — ' .$this->getName();
    }
    
    /**
     * 
     * @return boolean
     */
    public function isDeletableSafely() {
        return (count($this->clips) == 0);
    }
    
    /**
     * 
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getClipsNoDel() {
        return $this->getClips()->filter(function($p) {
            return $p->getDeleted() == false;
        });
    }

    /**
     * @Assert\Callback
     */
    public function isValidYoutubeURL(ExecutionContextInterface $context, $payload) {
        if($this->getUrl() == null) {
            return;
        }

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $this->getUrl());
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_exec($ch);
        $code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        $regex = [
            "/^((http|https):\/\/)?(www.)?youtube\.com\/watch\?v=[A-Za-z0-9_\-]+$/",
            "/^((http|https):\/\/)?youtu\.be\/[A-Za-z0-9_\-]+$/"
        ];
        
        $res=0;
        foreach($regex as $reg) {
            $res += preg_match($reg, $this->getUrl());
        }

        if($res == 0 || $code != 200) {
            $context->buildViolation('Vous devez indiquer une url Youtube existante.')
                    ->atPath('url')
                    ->addViolation();
        }
    }
}
